import React, { useState, useEffect, useRef, useMemo } from "react";
import { cn } from "@/lib/utils";
import { Button } from "@/components/ui/button";
import { Link, useLocation } from "wouter";
import { ThemeToggle } from "./ThemeToggle";
import { useTheme } from "@/contexts/ThemeContext";
import { useAuth } from "@/contexts/AuthContext";
// Still using the fullstack logo design as requested
const darkLogoPath = "/fullstack-logo-dark.webp";
const lightLogoPath = "/fullstack-logo-light.webp";
import {
  DropdownMenu,
  DropdownMenuContent,
  DropdownMenuItem,
  DropdownMenuTrigger,
} from "@/components/ui/dropdown-menu";
import { Sheet, SheetContent, SheetTrigger } from "@/components/ui/sheet";
import { Menu, X, ArrowRight } from "lucide-react";
import { Separator } from "@/components/ui/separator";

const Navbar: React.FC = () => {
  const [isScrolled, setIsScrolled] = useState(false);
  const { theme } = useTheme();
  const { isAuthenticated } = useAuth();
  const [, setLocation] = useLocation();
  const scrollListenerRef = useRef<((e: Event) => void) | null>(null);
  const lastScrollY = useRef(0);
  const scrollTimeout = useRef<NodeJS.Timeout | null>(null);

  // Use a throttled scroll handler for better performance
  useEffect(() => {
    // Optimize scroll event with throttling
    const handleScroll = () => {
      if (scrollTimeout.current) return;

      scrollTimeout.current = setTimeout(() => {
        // Only update state when crossing the threshold to minimize re-renders
        const currentScrollY = window.scrollY;
        if (
          (currentScrollY > 10 && lastScrollY.current <= 10) ||
          (currentScrollY <= 10 && lastScrollY.current > 10)
        ) {
          setIsScrolled(currentScrollY > 10);
        }

        lastScrollY.current = currentScrollY;
        scrollTimeout.current = null;
      }, 10);
    };

    // Store the handler reference for cleanup
    scrollListenerRef.current = handleScroll;

    // Use passive listener for better scroll performance
    window.addEventListener("scroll", handleScroll, { passive: true });

    return () => {
      if (scrollListenerRef.current) {
        window.removeEventListener("scroll", scrollListenerRef.current);
      }
      if (scrollTimeout.current) {
        clearTimeout(scrollTimeout.current);
      }
    };
  }, []);

  // Define the type for nav links
  interface NavLink {
    text: string;
    href: string;
    isPageLink?: boolean;
  }

  // Separate links into main and grouped links
  const mainNavLinks = useMemo<NavLink[]>(
    () => [
      { text: "Hjem", href: "/", isPageLink: true },
      { text: "Prosjekter", href: "/prosjekter", isPageLink: true },
      { text: "Kompetanse", href: "/kompetanse", isPageLink: true },
      { text: "Om meg", href: "/om-oss", isPageLink: true },
      { text: "Blogg", href: "/blogg", isPageLink: true },
      //{ text: "3D Palletizer", href: "/3d-palletizer-demo", isPageLink: true },
    ],
    [],
  );

  // Function to handle customer portal navigation based on auth status
  const handlePortalClick = (e: React.MouseEvent) => {
    e.preventDefault();
    if (isAuthenticated) {
      // If already authenticated, go directly to dashboard
      setLocation("/customer/dashboard");
    } else {
      // Otherwise go to login page
      setLocation("/login");
    }
  };

  return (
    <nav
      className={cn(
        "fixed top-0 left-0 w-full z-50 transition-all duration-300",
        // Always show solid background for better visibility
        theme === "dark"
          ? "bg-background/95 backdrop-blur-sm shadow-md border-b border-border/40"
          : "bg-white shadow-md",
        theme === "light" ? "text-gray-800" : "text-gray-100",
      )}
      style={{ height: "64px" }} /* Add explicit height to calculate offsets */
    >
      <div className="container mx-auto px-4 py-3 flex justify-between items-center">
        <Link href="/" className="flex items-center">
          <img
            src={theme === "dark" ? darkLogoPath : lightLogoPath}
            alt="Vetle Haakestad Logo"
            className="h-10 mr-3"
          />
          <span className="text-xl font-tech font-bold">
            <span className="text-primary">Vetle </span>
            <span className="text-foreground">Haakestad</span>
          </span>
        </Link>

        <div className="hidden md:flex space-x-6 items-center justify-center flex-1">
          <div className="flex items-center justify-center space-x-6 h-10">
            {/* Main navigation links */}
            {mainNavLinks
              .filter((link) => link.isPageLink || link.href === "/")
              .map((link, index) =>
                link.isPageLink ? (
                  <Link
                    key={index}
                    href={link.href}
                    className={cn(
                      "hover:text-primary transition-colors text-lg flex items-center h-full",
                      theme === "light" ? "text-gray-700" : "text-gray-300",
                    )}
                  >
                    {link.text}
                  </Link>
                ) : (
                  <a
                    key={index}
                    href={link.href}
                    className={cn(
                      "hover:text-primary transition-colors text-lg flex items-center h-full",
                      theme === "light" ? "text-gray-700" : "text-gray-300",
                    )}
                  >
                    {link.text}
                  </a>
                ),
              )}

            <ThemeToggle />
            {/* Customer portal button removed temporarily
            <div className="flex items-center h-full">
              <Button
                variant="outline"
                size="sm"
                className={cn(
                  "h-9",
                  theme === "light"
                    ? "border-primary bg-transparent text-primary hover:bg-primary/10 hover:text-primary-foreground"
                    : "text-white",
                )}
                onClick={handlePortalClick}
              >
                Kundeportal
              </Button>
            </div> */}
            <Link href="/kontakt" className="flex items-center h-full">
              <Button variant="glow" size="pill" className="h-9">
                Kontakt
              </Button>
            </Link>
          </div>
        </div>

        <div className="md:hidden flex items-center space-x-2">
          <ThemeToggle />

          {/* Mobile hamburger menu */}
          <Sheet>
            <SheetTrigger asChild>
              <Button
                variant="ghost"
                size="icon"
                className="relative min-h-[44px] min-w-[44px] p-0"
              >
                <Menu className="h-6 w-6" />
                <span className="sr-only">Åpne meny</span>
              </Button>
            </SheetTrigger>
            <SheetContent side="right" className="w-full sm:w-[300px] p-0">
              <div className="flex flex-col h-full">
                <div className="p-4 flex items-center justify-between">
                  <div className="flex items-center">
                    <img
                      src={theme === "dark" ? darkLogoPath : lightLogoPath}
                      alt="Vetle Haakestad Logo"
                      className="h-8 mr-2"
                    />
                    <span className="text-xl font-tech font-bold">
                      <span className="text-primary">Vetle</span>
                      <span className="text-secondary">Haakestad</span>
                    </span>
                  </div>
                  <ThemeToggle />
                </div>
                <Separator />

                <nav className="flex-1 overflow-auto p-4">
                  <div className="space-y-3">
                    {/* Main navigation links - filter out process and pricing links */}
                    {mainNavLinks
                      .filter((link) => link.isPageLink || link.href === "/")
                      .map((link, index) => (
                        <div key={index} className="w-full">
                          {link.isPageLink ? (
                            <Link
                              href={link.href}
                              className="flex items-center justify-center w-full py-3 px-4 rounded-md hover:bg-primary/10 hover:text-primary transition-colors duration-200 min-h-[44px] text-lg"
                            >
                              {link.text}
                            </Link>
                          ) : (
                            <a
                              href={link.href}
                              className="flex items-center justify-center w-full py-3 px-4 rounded-md hover:bg-primary/10 hover:text-primary transition-colors duration-200 min-h-[44px] text-lg"
                            >
                              {link.text}
                            </a>
                          )}
                        </div>
                      ))}
                  </div>
                </nav>

                <Separator />
                <div className="p-4 flex flex-col gap-3">
                  <div className="w-full">
                    <Button
                      variant="outline"
                      className={cn(
                        "w-full min-h-[44px] flex items-center justify-center",
                        theme === "light"
                          ? "border-primary bg-transparent text-primary hover:bg-primary/10 hover:text-primary-foreground"
                          : "text-white",
                      )}
                      onClick={handlePortalClick}
                    >
                      Kundeportal
                    </Button>
                  </div>
                  <Link href="/kontakt" className="w-full">
                    <Button
                      variant="glow"
                      className="w-full min-h-[44px] flex items-center justify-center"
                    >
                      Kontakt
                      <ArrowRight className="ml-2 h-4 w-4" />
                    </Button>
                  </Link>
                </div>
              </div>
            </SheetContent>
          </Sheet>
        </div>
      </div>
    </nav>
  );
};

export default Navbar;
